#  This script is compatible with MPC5604P device / Z0H core.
#  This script performs minimal intialization like ECC SRAM, IVORs.

##############################################
# Initialize a RAM range from 'start' address
##############################################
proc init_ram {start range} {
    # same registrer group name for all PowerPC
    variable GPR_GROUP "General Purpose Registers/"

    # load code start - start + 0x18
    set addr [expr $start + 0x18]
    reg ${GPR_GROUP}GPR11 %d = $addr
    #  actual range
    set c [expr ($range - 0x18)/128]
    reg ${GPR_GROUP}GPR12 %d = $c
    # execute init ram code
    #mtctr r12
    mem v:$start = 0x7D8903A6
    #stmw r0,0(r11)
    mem v:[format %x [expr $start + 0x4]] = 0x180B0900
    #addi r11,r11,128
    mem v:[format %x [expr $start + 0x8]] = 0x1D6B0080
    #bdnz -8
    mem v:[format %x [expr $start + 0xc]] = 0x7A20FFF8
    # init the rest of RAM
    #stmw   r6,0(r11)  - last 104 bytes of the range
    mem v:[format %x [expr $start + 0x10]] = 0x18CB0900
    # infinte loop
    #bl
    mem v:[format %x [expr $start + 0x14]] = 0x78000001
    
    # set PC to the first init instruction
    reg ${GPR_GROUP}PC = $start
    
    # execute init ram code
    # timeout 1 second
    go 1
    stop
}

#################################
# Initialization for e200Z0H core
#################################
proc init_e200z0h {} {
    #  ---------------------------------------------------------------------------------------
    #  Interrupt vectors IVOR 0-15, these SPRs are hardwired to specific
    # values, and are readable, but a mtspr results in an unimplemented or 
    # privileged exception
    # IVOR 16-31 are reserved
    #  ---------------------------------------------------------------------------------------
    variable e200SPR_GROUP "e200z0h Special Purpose Registers/"
    reg ${e200SPR_GROUP}IVPR = 0x40000000
}

#################################
# Initialization for MPC5604P
#################################
proc mpc5604P_init {} {
    reset hard

    # Explicitly stop the core
    stop
    
    # Disable SWT Watchdog Timer
    mem v:0xfff38010 = 0x0000c520
    mem v:0xfff38010 = 0x0000d928
    mem v:0xfff38000 = 0xff00000A    
  
    # core init
    init_e200z0h
    
    # initialize MPC5604P ECC SRAM 40K:  0x40000000 - 0x4000A000
    init_ram 0x40000000 0xA000
}

proc envsetup {} {
    # Environment Setup
    radix x 
    config hexprefix 0x
    config MemIdentifier v 
    config MemWidth 32 
    config MemAccess 32 
    config MemSwap off
}

#-------------------------------------------------------------------------------
# Main                                                                          
#-------------------------------------------------------------------------------

envsetup
# Note:  some 56xx connections provider may initializes the core upon connect, eg. PnE

mpc5604P_init
