//==============================================================================
// By R62780 Sep/3/2012
//==============================================================================
#include "MPC5604P.h"
#include <IntcInterrupts.h>
/*==============================================================================
 - PIT is clocked by SYSCLK
 
   Please select the board you use in the definition below (DEVELOPMENT BOARD
   DEFINITION):
   either:
   - tested on XPC56XX EVB MotherBoard + XPC560P 144LQFP Mini-Module
   MCU type: SPC5604PGVLQ6  OSCCLK=40MHz SYSCLK=64MHz (set by FMPLL)
   or 
   - tested also on TRK-MPC5604P
   MCU type: SPC5604PGMLQ6  OSCCLK=8MHz SYSCLK=64MHz  (set by FMPLL)


 - PIT1 has higher interrupt priority in order to be able to interrupt PIT0
   interrupt

 - In order to be able to visualize interrupt nesting; there is loop inside the
   PIT0_Isr function till new overflow flag is set. While the WHILE loop is 
   runningis within this interrupt subroutine it is interrupted by interrupt
   from PIT1_Isr.

 - In order to use nested interrupts (to be able to correctly enter and leave
   nested interrupt) do not forget to change following definition in the file
   IntcInterrupts.c:
   INTC_NESTED_INTERRUPT == 0 change to
   INTC_NESTED_INTERRUPT == 1

 - For RAM target ensure the INTC and MSR.EE are initialized immediately 
     after reset 
    
     In order to be the function usr_init called to be INTC (interrupt 
     controller) and MSR.EE bit initialized then one of the following 
     definitions must be defined in the C/C++ Preprocessor.
     
     a) under CW 10.2
       Go to the folder "Prefix" and open file MPC5604P_INT_RAM_VLE.prefix and
       add folowing line:

       #define CALL_USR_INIT 1
       
     
     b) under CW 2.10
      
       (Press Alt F7 -> C/C++ Preprocessor -> insert  #define CALL_USR_INIT 1 into
         Prefix text window ):
     
       #define CALL_USR_INIT 1
       or
       #define ROM_VERSION   1
       or
       #define CACHE_VERSION 1


       
WARNING_1: If your MCU uses 8MHz crystal (crystals less than 16MHz) then CMU_0 
           must be initialized differently from the default value.

WARNING_2: In the CW10.2 you have to correct the file ivor_branch_table.c.
       There is mistake. The correct line order is:
       
   a) either
   //=====================================================
   // function's prototypes
   #pragma section code_type ".ivor_branch_table"
   asm void ivor_branch_table_p1(void);
   //=====================================================
   // function's bodies
   #pragma section code_type ".ivor_branch_table"
   #pragma force_active on
   #pragma function_align 16

   asm void ivor_branch_table_p1(void) {
     nofralloc
     
   b) or  
   //=====================================================
   // function's prototypes
   #pragma section code_type ".ivor_branch_table"
   asm void ivor_branch_table_p1(void);
   //=====================================================
   // function's bodies
   #pragma force_active on
   #pragma function_align 16

   asm void ivor_branch_table_p1(void) {
     nofralloc
   
   c) or
       #pragma section code_type ".ivor_branch_table"

       #pragma force_active on

       #pragma function_align 16

       asm void ivor_branch_table_p1(void) {
       nofralloc
              
       
 - tested on XPC56XX EVB MotherBoard + XPC560P 144LQFP Mini-Module
   MCU type: SPC5604PGVLQ6
==============================================================================
==============================================================================*/

//==============================================================================
// DEVELOPMENT BOARD DEFINITION
//==============================================================================

//#define XPC56XXEVB_XPC560P_144LQFP_Mini-Module
#define TRK_MPC5604P

//==============================================================================
// Definitions
//==============================================================================

// Define IRQ#
#define PIT0 59
#define PIT1 60
#define PIT2 61
#define PIT3 127


// Define pads connected to LED1~4   (ports D4-D7)
#define LED1 52
#define LED2 53
#define LED3 54
#define LED4 55


// Define actions with  LEDs
#define LED1_ON   SIU.GPDO[LED1].R  = 0x00;  // LED1_ON
#define LED1_OFF  SIU.GPDO[LED1].R  = 0x01;  // LED1_OFF
#define LED1_TGL  SIU.GPDO[LED1].R ^= 0x01;  // LED1_TOGGLE

#define LED2_ON   SIU.GPDO[LED2].R  = 0x00;  // LED2_ON
#define LED2_OFF  SIU.GPDO[LED2].R  = 0x01;  // LED2_OFF
#define LED2_TGL  SIU.GPDO[LED2].R ^= 0x01;  // LED2_TOGGLE

#define LED3_ON   SIU.GPDO[LED3].R  = 0x00;  // LED3_ON
#define LED3_OFF  SIU.GPDO[LED3].R  = 0x01;  // LED3_OFF
#define LED3_TGL  SIU.GPDO[LED3].R ^= 0x01;  // LED3_TOGGLE

#define LED4_ON   SIU.GPDO[LED4].R  = 0x00;  // LED4_ON
#define LED4_OFF  SIU.GPDO[LED4].R  = 0x01;  // LED4_OFF
#define LED4_TGL  SIU.GPDO[LED4].R ^= 0x01;  // LED4_TOGGLE

//==============================================================================
// Function Prototypes
//==============================================================================
void SWT_Disable(void);

void InitModesAndClks(void); 

void PIT0_Init(void);
void PIT0_Run(void);
void PIT0_Isr(void); 

void PIT1_Init(void);
void PIT1_Isr(void); 
void PIT1_Run(void);

void LEDs_Init(void);
int  main(void);

/*******************************************************************************
Function Name : InitCMU_0
Engineer      : b08110
Date          : Dec-19-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : CMU_0 must be initialized differently from the default value
                in case of 8 MHz crystal.
Issues        : NONE
*******************************************************************************/
static void InitCMU_0(void)
{
   CGM.CMU_0_CSR.R = 0x00000004;    
}
//==============================================================================
// Enter RUN0 with FMPLL which generates SYSCLK = 64 MHz either 40 MHz OSCCLK.
// or from 8 MHz OSCCLK in dependence on the board definition.
//==============================================================================

void InitModesAndClks(void) 
{
 ME.MER.R          = 0x0000001D; // Enable DRUN, RUN0, SAFE, RESET modes

#ifdef TRK_MPC5604P  
 CGM.FMPLL[0].CR.R = 0x01200100; // 8MHz  xtal: Set PLL0 to 16 MHz
#else
 CGM.FMPLL[0].CR.R = 0x11200100; // 40MHz xtal: Set PLL0 to 64 MHz
#endif

 ME.RUNPC[0].R     = 0x000000FE; // enable peripherals run in all modes
 ME.RUN[0].R       = 0x001F0074; // RUN0 cfg: IRCON,OSC0ON,PLL0ON,syclk=PLL
 //.............................
 // Mode Transition to enter RUN0 mode:
 //.............................
 ME.MCTL.R         = 0x40005AF0; // Enter RUN0 Mode & Key
 ME.MCTL.R         = 0x4000A50F; // Enter RUN0 Mode & Inverted Key
 while(ME.GS.B.S_MTRANS) {} ;    // Wait for mode transition to complete
 while(ME.GS.B.S_CURRENTMODE != 4) {}; // Verify RUN0 is the current mode
}								

//==============================================================================
// PIT0_Init
//==============================================================================

void PIT0_Init(void)
{

 INTC_InstallINTCInterruptHandler(PIT0_Isr, PIT0, 1);

 PIT.PITMCR.R      = 0x01;       // Enable clk for PIT, stop PITs in debug mode
 
 PIT.CH[0].LDVAL.R = 64000000LU; // Timeout=LDVAL/SYSCLKS = 64,000,000/64M = 1s
 PIT.CH[0].TCTRL.R = 0x01;       // Freeze while Debug mode

 PIT.CH[0].TCTRL.B.TIE = 1;      // Enable interrupt from PIT0

}

//==============================================================================
// PIT0_Run
//==============================================================================

void PIT0_Run(void)
{
 PIT.CH[0].TCTRL.B.TEN = 1;      // start PIT0 counting
}

//==============================================================================
// PIT0_Isr
//==============================================================================

void PIT0_Isr(void) 
{
 PIT.CH[0].TFLG.R = 1;           // Clear PIT 0 flag by writing 1

 LED1_TGL;

 while(!PIT.CH[0].TFLG.R) {};    // wait entire period in loop only for 
                                 // debugging purposes in order to visualize
                                 // nesting from PIT1_Isr
}

//==============================================================================
// PIT1_Init
//==============================================================================

void PIT1_Init(void)
{
 INTC_InstallINTCInterruptHandler(PIT1_Isr, PIT1, 2);

 PIT.PITMCR.R      = 0x01;       // Enable clk for PIT, stop PITs in debug mode
 
 PIT.CH[1].LDVAL.R = 16000000LU; // Timeout=LDVAL/SYSCLKS= 16,000,000/64M= 0.25s
 PIT.CH[1].TCTRL.R = 0x01;       // Freeze while Debug mode


 PIT.CH[1].TCTRL.B.TIE = 1;      // Enable interrupt from PIT1

}

//==============================================================================
// PIT1_Run
//==============================================================================

void PIT1_Run(void)
{
 PIT.CH[1].TCTRL.B.TEN = 1;      // start PIT1 counting
}

//==============================================================================
// PIT1_Isr
//==============================================================================

void PIT1_Isr(void) 
{
 PIT.CH[1].TFLG.R = 1;         // Clear PIT 1 flag by writing 1

 LED2_TGL;
}

//==============================================================================
// disableWatchdog(void)
//==============================================================================

void SWT_Disable(void) 
{
 SWT.SR.R = 0x0000c520;          // Write keys to clear soft lock bit
 SWT.SR.R = 0x0000d928; 
 SWT.CR.R = 0x8000010A;          // Clear watchdog enable (WEN)
}

//==============================================================================
// LEDs_Init
//==============================================================================
void LEDs_Init(void)
{
  SIU.PCR[LED1].R = 0x0200;      // Configure pad 52 PD[4] as output
  SIU.PCR[LED2].R = 0x0200;      // Configure pad 53 PD[5] as output
  SIU.PCR[LED3].R = 0x0200;      // Configure pad 54 PD[6] as output
  SIU.PCR[LED4].R = 0x0200;      // Configure pad 55 PD[7] as output
  
  LED1_OFF;                      
  LED2_OFF;
  LED3_OFF;
  LED4_OFF;
}
//==============================================================================
//int main(void)
//==============================================================================
int main(void) 
{
  volatile int i = 0;

  SWT_Disable();                 // Disable watchdog
  
#ifdef TRK_MPC5604P  
  InitCMU_0();                   // 8 MHz OSC requires non-default CMU_0 init
#endif
  
  InitModesAndClks();            // Initialize mode entries
  LEDs_Init();
  PIT0_Init();
  PIT1_Init();
  PIT0_Run();
  PIT1_Run();


  // lower current INTC priority to start INTC interrupts
  INTC.CPR.R = 0;

  // Loop forever
  for (;;) 
    {
      i++;
    }
}
//==============================================================================





